Exception: Phronomy::Context::UnknownModelError
- Defined in:
- lib/phronomy/context/token_budget.rb
Overview
Raised when a model name is not found in the RubyLLM model registry and no explicit context_window was provided.
Raised when a model name is not found in the RubyLLM model registry and no explicit context_window was provided.