Class: Phronomy::Guardrail::OutputGuardrail
- Defined in:
- lib/phronomy/guardrail/output_guardrail.rb
Overview
Guardrail applied to agent/chain output before it is returned to the caller.
Guardrail applied to agent/chain output before it is returned to the caller.