Exception: Phronomy::InvalidAsyncTransitionActionError

Inherits:
InvalidAsyncWorkflowActionError show all
Defined in:
lib/phronomy/invalid_async_transition_action_error.rb

Overview

Raised when a synchronous Workflow transition action returns Phronomy::Task.

Transition actions are Run-to-Completion callbacks. They may start asynchronous work, but completion must return through a later explicit event.