Exception: Phronomy::PoolShutdownError

Inherits:
Error
  • Object
show all
Defined in:
lib/phronomy.rb

Overview

Raised when an operation is submitted to a BlockingAdapterPool that has already been shut down via BlockingAdapterPool#shutdown.