Class: Phronomy::Tracing::NullTracer
- Defined in:
- lib/phronomy/tracing/null_tracer.rb
Overview
Instance Method Summary collapse
-
#finish_span(span) ⇒ Object
Does nothing.
-
#start_span(name) ⇒ Object
Returns a minimal span object with the given name.
Methods inherited from Base
Instance Method Details
#finish_span(span) ⇒ Object
Does nothing. Explicit nil is equivalent to an empty method body; the mutation "remove nil" is accepted as it does not change observable behaviour. mutant:disable
22 |
# File 'lib/phronomy/tracing/null_tracer.rb', line 22 def finish_span(span, **) = nil |
#start_span(name) ⇒ Object
Returns a minimal span object with the given name.
17 |
# File 'lib/phronomy/tracing/null_tracer.rb', line 17 def start_span(name, **) = SpanStruct.new(name) |