Exception: Phronomy::WorkflowContextOwnershipError

Inherits:
Error
  • Object
show all
Defined in:
lib/phronomy.rb

Overview

Raised when a WorkflowContext field is mutated from a thread that does not own the context. Deliver asynchronous updates back to the Workflow as later events via Workflow#signal instead of mutating context from worker callbacks.