Class: Phronomy::Embeddings::Base
- Inherits:
-
Object
- Object
- Phronomy::Embeddings::Base
- Defined in:
- lib/phronomy/embeddings/base.rb
Overview
Abstract interface for embedding adapters.
Concrete implementations must override #embed and return a vector
as an +Array
Direct Known Subclasses
Instance Method Summary collapse
-
#embed(text) ⇒ Array<Float>
Embed the given text and return a vector representation.
Instance Method Details
#embed(text) ⇒ Array<Float>
Embed the given text and return a vector representation.
14 15 16 |
# File 'lib/phronomy/embeddings/base.rb', line 14 def (text) raise NotImplementedError, "#{self.class}#embed is not implemented" end |