Exception: Phronomy::PoolShutdownError
- Defined in:
- lib/phronomy.rb
Overview
Raised when an operation is submitted to a BlockingAdapterPool that has already been shut down via BlockingAdapterPool#shutdown.
Raised when an operation is submitted to a BlockingAdapterPool that has already been shut down via BlockingAdapterPool#shutdown.