Exception: Phronomy::WorkflowContextOwnershipError
- Defined in:
- lib/phronomy.rb
Overview
Raised when a WorkflowContext field is mutated from a thread that does not own the context (i.e. not the EventLoop dispatch thread). Only raised in EventLoop mode. Use +context.merge(...)+ to produce a new context, or deliver updates as +:child_completed+ event payloads.